package gov.va.med.pharmacy.peps.service.common.session;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionException;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.DefaultTransactionDefinition;

import gov.va.med.pharmacy.peps.common.vo.NationalSetting;
import gov.va.med.pharmacy.peps.common.vo.NationalSettingVo;
import gov.va.med.pharmacy.peps.common.vo.Role;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.domain.common.capability.NationalSettingDomainCapability;
import gov.va.med.pharmacy.peps.domain.common.capability.ReportDomainCapability;
import gov.va.med.pharmacy.peps.service.common.reports.ReportExportState;
import gov.va.med.pharmacy.peps.service.common.reports.ReportObserver;
import gov.va.med.pharmacy.peps.service.common.reports.ReportProcessStatus;


/**
 * This class implements the operations common to all report processing. Subclasses
 * should implement the doExport() method to include processing specific to a 
 * particular report.
 */
public abstract class AbstractReportExportProcess implements ReportExportProcess, Runnable {

    /**
     * The logger.
     */
    private static final Logger LOG = org.apache.log4j.Logger.getLogger(AbstractReportExportProcess.class);
    
    /**
     * Default size of the output file.
     */
    public static final int FILE_SIZE = 300;
    
    /**
     * Default user id.
     */
    private static final Long USER_ID = 42L;
    
    /**
     * Default report processing timeout.
     */
    public static final int TIMEOUT = 600;
    
    /**
     * Default thread delay value.
     */
    public static final int SLEEP = 5000;
    
    /**
     * Default error message.
     */
    public static final String ERROR = "Cound not open file for export: ";

    /**
     * True if the report is currently being processed; otherwise, false.
     */
    private volatile boolean running;
    
    /**
     * Represents the current state of report processing.
     */
    private volatile ReportExportState exportState;
    
    /**
     * A list of objects which should be notified if the
     * report processing state changes.
     */
    private List<ReportObserver> observers;
    
    /**
     * Set to true to enable debug functionality.
     */
    private boolean debug = false;

    /**
     * The reportDomainCapability.
     */
    private ReportDomainCapability reportDomainCapability;
    
    /**
     * The nationalSettingDomainCapability.
     */
    private NationalSettingDomainCapability nationalSettingDomainCapability;
    
    /**
     * The domainService.
     */
    private DomainService domainService;
    
    /**
     * The transactionManager.
     */
    private PlatformTransactionManager transactionManager;


    @Override
    public final void run() {
        LOG.debug("=============> Starting run() in  AbstractReportExportProcess <====================");
        initialize();
        runProcesses();

        notifyObservers();
        LOG.debug("=============> Exiting run() in AbstractReportExportProcess <=======================");

    }

    /** 
     * This is the runProcesses method 
     */
    private synchronized void runProcesses() {

        while (running) {
            try {
                LOG.debug("AbstractReportExportProcess process started...");
                resetValues();
                runTask();

                // if we are done with all exports then finish up
                if (this.getExportState().isExportComplete()) {
                    LOG.debug("all file Exports are completed!");
                }
            } catch (InterruptedException e) {
                LOG.debug("AbstractReportExportProcess. Report Generate Process Interrupted" + e);
                this.stopProcess();
            }

            notifyObservers();
        }

        this.stopProcess();
    }

    /** Reset state values for AbstractReportExportProcess. */
    private void resetValues() {
        ReportExportState state = this.getExportState();
        state.setExportComplete(false);
        state.setRecordCounter(0);
        state.setRunning(true);
        state.setReportProcessStatus(ReportProcessStatus.RUNNING);
    }

    /** Initialize process for AbstractReportExportProcess. */
    private void initialize() {
        LOG.debug("initializing report export process...");
        this.setExportState(new ReportExportState());
        running = true;
        getExportState().setReportProcessStatus(ReportProcessStatus.RUNNING);
        getExportState().setRunning(running);
        getExportState().setExportComplete(false);
        getExportState().setRecordCounter(0);
        getExportState().setRecordTotal(0);
        observers = new Vector<ReportObserver>();
    }

    /** Stop process for AbstractReportExportProcess. */
    public final synchronized void stopProcess() {
        LOG.debug("==>>>Report Export process stopped...");
        running = false;
        getExportState().setReportProcessStatus(ReportProcessStatus.STOPPED);
        getExportState().setRunning(false);
    }

    /** 
     * Run tasks within process for the AbstractReportExportProcess.
     * 
     * @throws InterruptedException thrown if process interrupted 
     */
    private final void runTask() throws InterruptedException {
        LOG.debug("starting AbstractReportExportProcess runTask()...");

        if (Thread.interrupted()) {
            throw new InterruptedException();
        }

        doExport();

        this.getExportState().setExportComplete(true);
        running = false;

    }

    
    /**
     * Do report-specific processing.
     * 
     * @throws InterruptedException thrown if process interrupted 
     */
    public abstract void doExport() throws InterruptedException;
    
    
    /**
     * getUser for ReportNdfProcessImpl
     * @return UserVo
     */
    private UserVo getUser() {        
        // Create the User
        UserVo user = new UserVo();
        user.setFirstName("FDBAuto");
        user.setLastName("AddProcess");
        user.setStationNumber("999");
        user.setUsername("FdbAutoAddProcess");
        user.setLocation("National");
        user.setId(USER_ID);

        /*set the Role of NATIONAL_SERVICE_MANAGER*/
        user.addRole(Role.PSS_PPSN_MANAGER);

        return user;
    }

    /**
     * Obtains the current date/time to set the date start or stop date value for AbstractReportExportProcess.
     * @return dateValue
     */
    protected final Date getDateTime() {
        DateFormat dateFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss", Locale.US);
        Date dateValue = new Date();

        try {
            LOG.debug("AbstractReportExportProcess:getDateTime: " + dateFormat.format(dateValue));
        } catch (Exception e) {
            LOG.error("Exception parsing Start Date/Time: " + e.getMessage());
        }

        return dateValue;
    }

    /**
     * The setDateValue method.
     * @param type type
     * @param date date
     */
    protected final void setDateValue(NationalSetting type, Date date) {
        try {
            DefaultTransactionDefinition def = new DefaultTransactionDefinition(TransactionDefinition.PROPAGATION_REQUIRES_NEW);
            def.setTimeout(TIMEOUT);
            TransactionStatus status = transactionManager.getTransaction(def);
            List<NationalSettingVo> settingsList = nationalSettingDomainCapability.retrieve();

            // iterate through all the nationalsettings looking for the type
            for (NationalSettingVo vo : settingsList) {
                if (type.toString().equals(vo.getKeyName())) {
                    vo.setDateValue(date);
                    nationalSettingDomainCapability.update(vo, getUser());
                }
            }

            transactionManager.commit(status);

        } catch (TransactionException e) {
            LOG.error("TransactionException, in setDateValue()  " + e.getMessage());

            if (e.getCause() != null) {
                LOG.error("The Underlying cause is " + e.getCause().getMessage());
            }
        } catch (Exception e) {
            LOG.error("Exception getting setDateValue " + type + ":" + e.getMessage());
        }

    }

    /**
     * registerObserver in AbstractReportExportProcess.
     * @param pObserver pObserver
     */
    @Override
    public final void registerObserver(ReportObserver pObserver) {
        observers.add(pObserver);

    }

    /**
     * removeObserver in AbstractReportExportProcess.
     *  @param pObserver pObserver
     */
    @Override
    public final void removeObserver(ReportObserver pObserver) {
        observers.remove(pObserver);

    }

    /**
     * notifyObservers in AbstractReportExportProcess.
     */
    @Override
    public final void notifyObservers() {
        for (int loopIndex = 0; loopIndex < observers.size(); loopIndex++) {
            ReportObserver observer = (ReportObserver) observers.get(loopIndex);
            observer.update(this.getExportState());
        }
    }

    /**
     * Get export state in AbstractReportExportProcess.
     * @return the exportState
     */
    public final ReportExportState getExportState() {
        return exportState;
    }

    /**
     * Set export state in AbstractReportExportProcess.
     * @param exportState the exportState to set
     */
    public final void setExportState(ReportExportState exportState) {
        this.exportState = exportState;
    }

    /**
     * getRecordCounter in AbstractReportExportProcess.
     * @return zero
     */
    @Override
    public final int getRecordCounter() {
        return 0;
    }

    @Override
    public final void setReportProcessStatus(ReportProcessStatus reportProcessStatus) {
    }

    /**
     * getReportDomainCapability in AbstractReportExportProcess.
     * @return reportDomainCapability ReportDomainCapability
     */
    public final ReportDomainCapability getReportDomainCapability() {
        return reportDomainCapability;
    }

    /**
     * Sets reportDomainCapability in AbstractReportExportProcess.
     * @param reportDomainCapability reportDomainCapability
     */
    public final void setReportDomainCapability(ReportDomainCapability reportDomainCapability) {
        this.reportDomainCapability = reportDomainCapability;
    }

    /**
     * getNationalSettingDomainCapability in AbstractReportExportProcess.
     * @return nationalSettingDomainCapability 
     */
    public final NationalSettingDomainCapability getNationalSettingDomainCapability() {
        return nationalSettingDomainCapability;
    }

    /**
     * setNationalSettingDomainCapability in AbstractReportExportProcess.
     * @param nationalSettingDomainCapability nationalSettingDomainCapability
     */
    public final void setNationalSettingDomainCapability(NationalSettingDomainCapability nationalSettingDomainCapability) {
        this.nationalSettingDomainCapability = nationalSettingDomainCapability;
    }

    /**
     * getDomainService in AbstractReportExportProcess.
     * @return domainService
     */
    public final DomainService getDomainService() {
        return domainService;
    }

    /**
     * setDomainService in AbstractReportExportProcess.
     * @param domainService domainService
     */
    public final void setDomainService(DomainService domainService) {
        this.domainService = domainService;
    }

    /**
     * getTransactionManager in AbstractReportExportProcess.
     * @return transactionManager
     */
    public final PlatformTransactionManager getTransactionManager() {
        return transactionManager;
    }

    /**
     * setTransactionManager in AbstractReportExportProcess.
     * @param transactionManager transactionManager
     */
    public final void setTransactionManager(PlatformTransactionManager transactionManager) {
        this.transactionManager = transactionManager;
    }

    public final boolean isDebug() {
        return debug;
    }

    public final void setDebug(boolean debug) {
        this.debug = debug;
    }
}
