package gov.va.med.pharmacy.peps.service.common.session.impl;

import java.util.Date;
import java.util.List;

import org.apache.log4j.Logger;
import org.springframework.transaction.TransactionException;

import gov.va.med.pharmacy.peps.common.vo.NationalSetting;
import gov.va.med.pharmacy.peps.common.vo.ReportCopayTierChangeVo;
import gov.va.med.pharmacy.peps.service.common.reports.ReportExportState;
import gov.va.med.pharmacy.peps.service.common.reports.ReportProcessStatus;
import gov.va.med.pharmacy.peps.service.common.session.AbstractReportExportProcess;
import gov.va.med.pharmacy.peps.service.common.session.ReportCopayTierChangeProcess;
import gov.va.med.pharmacy.peps.service.common.utility.ReportCopayTierChangeCsvFile;


/**
 * This class retrieves the data for the 'Products with Changes to Copay Tier Data'
 * report and creates a caret separated value file for download.
 */
public class ReportCopayTierChangeProcessImpl extends AbstractReportExportProcess implements ReportCopayTierChangeProcess {
    /**
     * Log4j logger.
     */
    private static final Logger LOG = org.apache.log4j.Logger.getLogger(ReportCopayTierChangeProcessImpl.class);
    
    /**
     * The report will return products where the associated copay tier information has
     * changed on or after startDate. If startDate is null, all dates will be returned. 
     */
    private Date startDate;
    
    /**
     * Numeric flag indicating whether active products should be
     * included in the report. A value of 1 will include active products;
     * a value of 0 will exclude active products.
     */
    private int includeActiveProducts;
    
    /**
     * Numeric flag indicating whether inactive products should be
     * included in the report. A value of 1 will include inactive products;
     * a value of 0 will exclude inactive products.
     */
    private int includeInactiveProducts;
    
    /**
     * The value used to separate multiple drug class codes for the same
     * product.
     */
    private String drugClassDelimiter;
    
    /**
     * Constructor.
     */
    public ReportCopayTierChangeProcessImpl() {
        super();
        includeActiveProducts = 1;
        includeInactiveProducts = 1;
        drugClassDelimiter = "|";
    }
    
    @Override
    public final void doExport() throws InterruptedException {
        LOG.debug("Exporting Product with Copay Tier Changes Report");

        boolean errorOccurred = false;
        this.getExportState().setReportProcessStatus(ReportProcessStatus.RUNNING);
        ReportExportState state = this.getExportState();        

        try {
            //Set start time
            setDateValue(NationalSetting.REPORT_COPAY_TIER_CHANGE_START, getDateTime());
            
            ReportCopayTierChangeCsvFile csv = new ReportCopayTierChangeCsvFile();

            try {
                List<ReportCopayTierChangeVo> list = getReportDomainCapability().getCopayTierChanges(
                    startDate,
                    includeActiveProducts,
                    includeInactiveProducts,
                    drugClassDelimiter);
                state.setRecordTotal(list.size());
                csv.createFile(list, state);
            } catch (Exception e) {
                LOG.error(ERROR, e);
                errorOccurred = true;
            }
        } catch (TransactionException e) {
            LOG.error("FAILED TO retrieve List. Error Message is " + e.getMessage(), e);
            errorOccurred = true;            
        }
        finally {
            if(!errorOccurred) {
                //Set stop time
                setDateValue(NationalSetting.REPORT_COPAY_TIER_CHANGE_COMPLETE, getDateTime());                
                this.getExportState().setReportProcessStatus(ReportProcessStatus.COMPLETED);
            }
            else {
                this.stopProcess();
            }
        }
    }

    @Override
    public final Date getStartDate() {
        return startDate;
    }

    @Override
    public final void setStartDate(Date startDate) {
        this.startDate = startDate;
    }

    @Override
    public final boolean getIncludeActiveProducts() {
        return includeActiveProducts == 1;
    }

    @Override
    public final void setIncludeActiveProducts(boolean includeActiveProducts) {
        if(includeActiveProducts) {
            this.includeActiveProducts = 1;
        }
        else {
            this.includeActiveProducts = 0;
        }
    }

    @Override
    public final boolean getIncludeInactiveProducts() {
        return includeInactiveProducts == 1;
    }

    @Override
    public final void setIncludeInactiveProducts(boolean includeInactiveProducts) {
        if(includeInactiveProducts) {
            this.includeInactiveProducts = 1;
        }
        else {
            this.includeInactiveProducts = 0;
        }
    }

    @Override
    public final String getDrugClassDelimiter() {
        return drugClassDelimiter;
    }

    @Override
    public final void setDrugClassDelimiter(String drugClassDelimiter) {
        this.drugClassDelimiter = drugClassDelimiter;
    }

}
