/*
 * Created on Jul 13, 2005
 *
 * $Id: BlankLinesCheck.java,v 1.1 2007/03/12 16:51:01 tstavenger Exp $
 */


package gov.va.med.pharmacy.checkstyle.checks;import com.puppycrawl.tools.checkstyle.api.Check;
import com.puppycrawl.tools.checkstyle.api.DetailAST;
import com.puppycrawl.tools.checkstyle.api.TokenTypes;



/**
 * A Checkstyle Check that checks that blank lines exist
 * between various blocks of code (see Section 4.1 Blank Lines 
 * of the SwRI / VA PRE Coding Standards v1.3).
 * 
 * @author kvisco
 * @version $Revision: 1.1 $ $Date: 2007/03/12 16:51:01 $
 */
public class BlankLinesCheck extends Check {

    private static final String BLANK_LINE_AFTER_JAVADOC = "Invalid blank line between Javadoc and ''{0}''";

    private static final String MISSING_BLANK_LINE_AFTER = "Missing a required blank line after ''{0}''.";

    private static final String MISSING_BLANK_LINE_BEFORE = "Missing a required blank line before ''{0}''.";

    private static final String ELSE_KEYWORD = "else";
    private static final String IMPORT_KEYWORD = "import";

    private int numBlankLinesBeforePackageDef = 2;
    private int numBlankLinesAfterPackageDef = 2;
    private int numBlankLinesAfterImports = 2;

    /**
     * Creates a new instance of the BlankLinesCheck
     */
    public BlankLinesCheck() {
        super();
    }

    /* (non-Javadoc)
     * @see com.puppycrawl.tools.checkstyle.api.Check#getDefaultTokens()
     */
    public int[] getDefaultTokens() {
        return new int[] {TokenTypes.CLASS_DEF, TokenTypes.INTERFACE_DEF, TokenTypes.METHOD_DEF, TokenTypes.CTOR_DEF,
                          TokenTypes.SUPER_CTOR_CALL, TokenTypes.CTOR_CALL, TokenTypes.LITERAL_RETURN,
                          TokenTypes.PACKAGE_DEF, TokenTypes.IMPORT, TokenTypes.LITERAL_IF, TokenTypes.LITERAL_FOR,
                          TokenTypes.LITERAL_TRY, TokenTypes.LITERAL_WHILE, TokenTypes.LITERAL_DO,};
    }

    /*
     *  (non-Javadoc)
     * @see com.puppycrawl.tools.checkstyle.api.Check#getRequiredTokens()
     */
    public int[] getRequiredTokens() {
        return getDefaultTokens();
    }

    /*
     *  (non-Javadoc)
     * @see com.puppycrawl.tools.checkstyle.api.Check#visitToken(com.puppycrawl.tools.checkstyle.api.DetailAST)
     */
    public void visitToken(DetailAST aAST) {

        // line numbers from DetailAST are from
        int currentLine = aAST.getLineNo() - 1;
        String[] lines = getLines();
        switch (aAST.getType()) {
            case TokenTypes.METHOD_DEF:
            case TokenTypes.CLASS_DEF:
            case TokenTypes.CTOR_DEF:
            case TokenTypes.INTERFACE_DEF:
                checkMethodOrClassDef(aAST, lines, currentLine);
                break;
            case TokenTypes.IMPORT:
                checkBlankLinesAfterImports(aAST, lines, currentLine);
                break;
            case TokenTypes.SUPER_CTOR_CALL:
            case TokenTypes.CTOR_CALL:
//                System.err.println("last child token: " + TokenTypes.getTokenName(aAST.getLastChild().getType()));
                if (aAST.getLastChild().getType() == TokenTypes.SEMI
                    && aAST.getNextSibling().getType() != TokenTypes.RCURLY) {
                    /*
                     * Find the semicolon of the "super(..);" or "this(..);" call, 
                     * check that the next line is blank.  This allows for formatting/splitting
                     * of the parameters onto new lines (which originally this carped about).
                     */
                    aAST = aAST.getLastChild();
                    currentLine = aAST.getLineNo() - 1;
                    lines = getLines();
                    checkNextLine(lines, currentLine);
                }
                else if (aAST.getNextSibling().getType() != TokenTypes.RCURLY) {
                    checkNextLine(lines, currentLine);
                }
                break;
            case TokenTypes.PACKAGE_DEF:
                if (currentLine > 1) {
                    // check # of lines before package def
                    int minIdx = currentLine - (numBlankLinesBeforePackageDef - 1);
                    int maxIdx = currentLine + 1;
                    for (int lineIdx = minIdx; lineIdx < maxIdx; lineIdx++) {
                        checkPreviousLine(lines, lineIdx);
                    }
                    // check # of lines after package def
                    minIdx = currentLine;
                    maxIdx = currentLine + numBlankLinesAfterPackageDef;
                    for (int lineIdx = minIdx; lineIdx < maxIdx; lineIdx++) {
                        checkNextLine(lines, lineIdx);
                    }
                }
                else {
                    log(currentLine + 1, MISSING_BLANK_LINE_BEFORE, lines[currentLine]);
                }
                break;
            case TokenTypes.LITERAL_IF:
                String ifstmt = lines[currentLine];
                // make sure we only handle "if" and
                // ignore "else if" stmts
                if (!ifstmt.trim().startsWith(ELSE_KEYWORD)) {
                    if (aAST.getPreviousSibling() != null) {
                        checkPreviousLine(lines, currentLine);
                    }
                }
                break;
            case TokenTypes.LITERAL_FOR:
            case TokenTypes.LITERAL_TRY:
            case TokenTypes.LITERAL_RETURN:
            case TokenTypes.LITERAL_WHILE:
            case TokenTypes.LITERAL_DO:
                if (aAST.getPreviousSibling() != null) {
                    checkPreviousLine(lines, currentLine);
                }

                break;
            default:
                break;
        }
    }

    /*
     *  (non-Javadoc)
     * @see com.puppycrawl.tools.checkstyle.api.Check#leaveToken(com.puppycrawl.tools.checkstyle.api.DetailAST)
     */
    public void leaveToken(DetailAST aAST) {
        String[] lines = getLines();
        switch (aAST.getType()) {
            case TokenTypes.LITERAL_FOR:
            case TokenTypes.LITERAL_TRY:
            case TokenTypes.LITERAL_WHILE:
            case TokenTypes.LITERAL_DO:
            case TokenTypes.LITERAL_IF:
                if ((aAST.getNextSibling() != null) && (aAST.getNextSibling().getType() != TokenTypes.RCURLY)) {
                    checkPreviousLine(lines, ((DetailAST) aAST.getNextSibling()).getLineNo() - 1);
                }
                break;
        }
    }

    /**
     * Checks that the last import contains the proper number of
     * blank lines following it
     * 
     * @param aAST the current DetailAST
     * @param lines the String array of each line in the file
     * @param current the current index into the array to begin checking
     */
    private void checkBlankLinesAfterImports(DetailAST aAST, String[] lines, int current) {
        int numBlankLines = 0;
        String line = lines[current];

        int lineNo = current + 1;
        for (; lineNo < lines.length; lineNo++) {
            line = lines[lineNo];
            if (isWhitespace(line)) {
                ++numBlankLines;
            }
            else if (line.trim().startsWith(IMPORT_KEYWORD)) {
                return;
            }
            else {
                break;
            }
        }

        if (numBlankLines == 0) {
            log(current + 1, MISSING_BLANK_LINE_AFTER, lines[current]);

        }
        else if (numBlankLines < numBlankLinesAfterImports) {
            log(lineNo + 1, MISSING_BLANK_LINE_BEFORE, line);
        }
    }

    /**
     * Checks that each method and each class definition are separated
     * by a blank line
     * 
     * @param aAST the current DetailAST
     * @param lines the String array of each line in the file
     * @param current the current index into the array to begin checking
     */
    private void checkMethodOrClassDef(DetailAST aAST, String[] lines, int current) {
        int currentLine = aAST.getLineNo();
        if (currentLine > 1) {
            int lineNo = currentLine - 1;

            // strip off any single line comments immediately before
            // method or class
            while ((lineNo > 0) && isSingleLineComment(lines[lineNo - 1])) {
                --lineNo;
            }

            // check to see if current line is whitespace
            boolean whitespace = ((lineNo > 0) && (isWhitespace(lines[lineNo - 1])));

            // check for Javadoc or block comments
            boolean foundBlockCommentEnd = false;
            boolean foundNonComment = false;

            while ((lineNo > 0) && (!(foundBlockCommentEnd = isBlockCommentEnd(lines[lineNo - 1])))) {
                if ((!isWhitespace(lines[lineNo - 1])) && (!isSingleLineComment(lines[lineNo - 1]))) {
                    foundNonComment = true;
                    break;
                }
                --lineNo;
            }

            if (foundNonComment && (!whitespace)) {
                log(lineNo + 1, MISSING_BLANK_LINE_BEFORE, lines[lineNo]);
            }

            if (foundBlockCommentEnd) {

                if (whitespace) {
                    log(lineNo + 1, BLANK_LINE_AFTER_JAVADOC, lines[current]);
                }
                while ((lineNo > 0) && (!isBlockCommentStart(lines[lineNo - 1]))) {
                    --lineNo;
                }
                if (lineNo > 0) {
                    --lineNo;
                }
            }
            if ((lineNo > 0) && (!isWhitespace(lines[lineNo - 1])) && (foundBlockCommentEnd)) {
                log(lineNo + 1, MISSING_BLANK_LINE_BEFORE, lines[lineNo]);
            }
        }
    }

    /**
     * Checks to make sure the previous line is blank and logs
     * an error if it is not. Single line comments are ignored s
     * such that one or more single line comments may appear directly
     * before the current line with the blank line preceeding the
     * comments.
     * 
     * @param lines the String array of each line in the file
     * @param current the current index into the array to begin checking
     */
    private void checkPreviousLine(String[] lines, int current) {
        if (current > 0) {
            int lineNo = current - 1;
            while ((lineNo > 0) && (isSingleLineComment(lines[lineNo]))) {
                --lineNo;
            }
            if (lineNo > 0) {
                if (!isWhitespace(lines[lineNo])) {
                    log(current + 1, MISSING_BLANK_LINE_BEFORE, lines[lineNo + 1]);
                }

            }
            else {
                log(current + 1, MISSING_BLANK_LINE_BEFORE, lines[current]);
            }
        }
        else {
            log(current + 1, MISSING_BLANK_LINE_BEFORE, lines[current]);
        }

    }

    /**
     * Checks to make sure the next line is blank and logs
     * an error if it is not.
     * 
     * @param lines the String array of each line in the file
     * @param current the current index into the array to begin checking
     */
    private void checkNextLine(String[] lines, int current) {
        if (current + 1 >= lines.length) {
            log(current + 1, MISSING_BLANK_LINE_AFTER, lines[current]);
        }
        else {
            if (!isWhitespace(lines[current + 1])) {
                log(current + 1, MISSING_BLANK_LINE_AFTER, lines[current]);
            }
        }
    }

    private boolean isBlockCommentEnd(String value) {
        if (value == null)
            return false;
        return value.trim().startsWith("*/");
    }

    private boolean isBlockCommentStart(String value) {
        if (value == null)
            return false;
        return value.trim().startsWith("/*");
    }

    private boolean isSingleLineComment(String value) {
        if (value == null)
            return false;
        return value.trim().startsWith("//");
    }

    private boolean isWhitespace(String value) {
        if (value != null) {
            char[] chars = value.toCharArray();
            for (int i = 0; i < chars.length; i++) {
                switch (chars[i]) {
                    case ' ':
                    case '\t':
                    case '\n':
                    case '\r':
                        break;
                    default:
                        return false;
                }
            }
            return true;
        }
        return false;
    }

}
