/*
 * Created on Jul 13, 2005
 *
 * $Id: HeaderCheck.java,v 1.1 2007/03/12 16:51:01 tstavenger Exp $
 */


package gov.va.med.pharmacy.checkstyle.checks;import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.puppycrawl.tools.checkstyle.api.Check;
import com.puppycrawl.tools.checkstyle.api.DetailAST;



/**
 * A Checkstyle Check that checks the code header to make sure it
 * adheres to the VA PRE Coding Standards.
 * 
 * @author kvisco
 * @version $Revision: 1.1 $ $Date: 2007/03/12 16:51:01 $
 */
public class HeaderCheck extends Check {

    private static final String[] EMPTY_STRING_ARRAY = new String[0];

    /**
     * A set of tokens that will cause the header check to abort
     */
    private static final String[] stopTokens = {"package", "import", "public", "protected", "private", "class", "interface"};

    /**
     * The starting lines of the header
     */
    private String[] startLines = EMPTY_STRING_ARRAY;

    /**
     * If extra lines are allowed in the header, these are
     * the lines that must exist (otherwise just use start +
     * end)
     */
    private String[] mustContain = EMPTY_STRING_ARRAY;

    /**
     * The ending lines of the header
     */
    private String[] endLines = EMPTY_STRING_ARRAY;

    /**
     * Allow extra "unmatched" lines in the header
     */
    private boolean allowExtraLines = true;

    /**
     * Creates a new instance of the HeaderCheck
     */
    public HeaderCheck() {
        super();
    }

    /* (non-Javadoc)
     * @see com.puppycrawl.tools.checkstyle.api.Check#getDefaultTokens()
     */
    public int[] getDefaultTokens() {
        return new int[0];
    }

    /**
     * Returns true if extra lines are allowed in the header,
     * otherwise false.
     * 
     * @return true if extra lines are allowed in the header
     */
    public boolean isAllowExtraLines() {
        return allowExtraLines;
    }

    /**
     * 
     * @param allowExtraLines
     */
    public void setAllowExtraLines(boolean allowExtraLines) {
        this.allowExtraLines = allowExtraLines;
    }

    /**
     * Set the end of the header to check against. 
     * Individual lines in the header must be separated by '\n' characters.
     * 
     * @param endHeader the ending of the header content to check against.
     */
    public void setEndHeader(String endHeader) {
        endLines = getLines(endHeader);
    }

    /**
     * Sets a set of lines which must be contained within the header.
     * Individual lines in the header must be separated by '\n' characters.
     * 
     * @param lines the lines of the header content to check against.
     */
    public void setMustContain(String lines) {
        mustContain = getLines(lines);
    }

    /**
     * Set the start of the header to check against. 
     * Individual lines in the header must be separated by '\n' characters.
     * 
     * @param startHeader the beginning of the header content to check against.
     */
    public void setStartHeader(String startHeader) {
        startLines = getLines(startHeader);
    }

    /*
     *  (non-Javadoc)
     * @see com.puppycrawl.tools.checkstyle.api.Check#beginTree(com.puppycrawl.tools.checkstyle.api.DetailAST)
     */
    public void beginTree(DetailAST aRootAST) {
        String[] lines = getLines();
        int offset = 0;
        if (!checkStartHeader(lines, offset))
            return;
        offset = startLines.length;
        CheckResult result = findMustContain(lines, offset);
        if (!result.matches)
            return;
        offset = result.stop;
        checkEndHeader(lines, offset);

    }

    private boolean checkStartHeader(String[] lines, int offset) {
        for (int s = 0; s < startLines.length; s++) {
            if (s + offset < lines.length) {
                if (!startLines[s].equals(lines[s + offset])) {
                    if (s == 0) {
                        log(offset + 1, "Missing or invalid source header.");
                    }
                    else {
                        log(s + offset + 1, "This line does not match expected header line of ''{0}''.", startLines[s]);
                    }
                }
            }
            else {
                log(s + offset + 1, "This line does not match expected header line of ''{0}''.", startLines[s]);
                return false;
            }
        }
        return true;
    }

    private CheckResult findMustContain(String[] lines, int offset) {
        CheckResult result = new CheckResult();
        result.start = offset;
        if ((mustContain != null) && (mustContain.length > 0)) {
            HashSet mustContainSet = new HashSet(mustContain.length);
            for (int i = 0; i < mustContain.length; i++) {
                mustContainSet.add(mustContain[i]);
            }
            for (int i = offset; i < lines.length; i++) {
                if (checkLine(mustContainSet, lines[i])) {
                    //                if (mustContainSet.remove(lines[i])) {
                    if (mustContainSet.isEmpty()) {
                        break;
                    }
                }
                else {
                    if (startsWithStopToken(lines[i]))
                        break;
                    if (!allowExtraLines) {
                        log(i + 1, "The following line should not appear in the header: {0}.", lines[i]);
                    }
                }
            }
            if (!mustContainSet.isEmpty()) {
                log(offset + 1, "The header does not contain the required line of ''{0}''.", mustContainSet.iterator()
                    .next());
            }
        }
        return result;
    }

    private boolean checkLine(HashSet containSet, String line) {
        Iterator iter = containSet.iterator();

        while (iter.hasNext()) {
            String curStr = (String) iter.next();
            Pattern current = Pattern.compile(curStr);
            Matcher m = current.matcher(line);
            if (m.matches()) {
                containSet.remove(curStr);
                return true;
            }
        }

        return false;
    }

    private boolean checkEndHeader(String[] lines, int offset) {
        int skip = offset;
        int eIdx = 0;
        while (eIdx < endLines.length) {
            if (eIdx + skip < lines.length) {
                if (!endLines[eIdx].equals(lines[eIdx + skip])) {
                    if (startsWithStopToken(lines[eIdx + skip])) {
                        log(eIdx + offset + 1, "This line does not match expected header line of ''{0}''.", endLines[eIdx]);
                        return false;
                    }
                    ++skip;
                    eIdx = 0;
                    continue;
                }
            }
            else {
                log(eIdx + offset + 1, "This line does not match expected header line of ''{0}''.", endLines[eIdx]);
                return false;
            }
            ++eIdx;
        }
        return true;
    }

    private String[] getLines(String value) {
        if ((value == null) || (value.trim().length() == 0)) {
            return EMPTY_STRING_ARRAY;
        }
        StringTokenizer tokens = new StringTokenizer(value, "\n");
        ArrayList list = new ArrayList();
        while (tokens.hasMoreElements()) {
            list.add(tokens.nextElement());
        }
        String[] lines = new String[list.size()];
        list.toArray(lines);
        return lines;
    }

    private boolean startsWithStopToken(String value) {
        if (value == null)
            return false;
        for (int i = 0; i < stopTokens.length; i++) {
            if (value.trim().startsWith(stopTokens[i])) {
                return true;
            }
        }
        return false;
    }


    private class CheckResult {
        int start;
        int stop;
        boolean matches = true;
    }
}
